@@ -65,13 +65,16 @@ class Database:
 
             elif freq == '1D':
                 df['datetime'] = df['date']
+                df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
+                cols = ['datetime', 'date', 'open', 'high', 'low', 'close', 'volume']
 
+            elif freq == '1DW':
+                df['datetime'] = df['date']
                 df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d')
                 cols = ['datetime','date','open','high','low','close','volume','susp','auc_close','adj_close','gross_volume','turnover','VWAP']
                 df['susp'] = df['susp'].astype(bool)
 
             df = df[cols]
-
             df = df.set_index(keys='datetime')
             df = df.sort_index()
 
@@ -86,7 +89,7 @@ class Database:
                 qfq_df = qfq_df.set_index(keys='date')
                 t_index = pd.DatetimeIndex(pd.date_range(start=first_date, end=last_date, freq='1D'))
                 qfq_df = qfq_df.reindex(t_index)
-                qfq_df = qfq_df.fillna(method='ffill')
+                qfq_df = qfq_df.fillna(method='bfill')
                 qfq_df = qfq_df.reset_index()
                 qfq_df['index'] = qfq_df['index'].astype(str)
                 qfq_df['index'] = qfq_df['index'].str.replace('-', '')
@@ -102,13 +105,16 @@ class Database:
                 df = df.set_index(keys='datetime')
 
                 if vol_adj == True:
-                    col_list = ['open', 'high', 'low', 'close']
+                    if freq == '1DW':
+                        col_list = ['open', 'high', 'low', 'close', 'volume','gross_volume']
+                    else:
+                        col_list = ['open', 'high', 'low', 'close', 'volume']
                 else:
-                    col_list = ['open', 'high', 'low', 'close', 'volume','gross_volume']
+                    col_list = ['open', 'high', 'low', 'close']
 
                 for col in col_list:
                     df[col] = df[col] * df['qfq_factor']
-                    if col != 'volume':
+                    if 'volume' not in col:
                         df[col] = df[col].map(lambda x: round(x, 3))
                     else:
                         df[col] = df[col].map(lambda x: round(x, 0))
@@ -117,7 +123,10 @@ class Database:
                 if 'T' in freq:
                     df = df[['date', 'time', 'open', 'high', 'low', 'close', 'volume']]
                 else:
-                    df = df[['date','open','high','low','close','volume','susp','auc_close','adj_close','gross_volume','turnover','VWAP']]
+                    if freq == '1DW':
+                        df = df[['date','open','high','low','close','volume','susp','auc_close','gross_volume','turnover','VWAP']]
+                    elif freq == '1D':
+                        df = df[['date','open','high','low','close','volume']]
 
             return df
     
@@ -1092,7 +1101,7 @@ class Database:
     #########################################################################################
 
     def check_hk_stock_ohlc_args(self, code, start_date, end_date, freq):
-        freq_list = ['1T', '5T', '15T', '30T', '1D']
+        freq_list = ['1T', '5T', '15T', '30T', '1D','1DW']
         freq_valid = True if freq in freq_list else False
 
         try:
@@ -1698,3 +1707,35 @@ def get_hk_holiday_and_expiry_date(start_year, end_year, format='int'):
     dict1 = {'expiry_date': expiry_date_list, 'public_holiday': holiday_list, 'half_day_mkt': half_day_mkt_list}
     return dict1
 
+def get_stock_tick_size(market, price):
+    if market == 'HK':
+        if price >= 0.01 and price < 0.25:
+            tick_size = 0.001
+        elif price >= 0.25 and price < 0.5:
+            tick_size = 0.005
+        elif price >= 0.5 and price < 10:
+            tick_size = 0.01
+        elif price >= 10 and price < 20:
+            tick_size = 0.02
+        elif price >= 20 and price < 100:
+            tick_size = 0.05
+        elif price >= 100 and price < 200:
+            tick_size = 0.1
+        elif price >= 200 and price < 500:
+            tick_size = 0.2
+        elif price >= 500 and price < 1000:
+            tick_size = 0.5
+        elif price >= 1000 and price < 2000:
+            tick_size = 1
+        elif price >= 2000 and price < 5000:
+            tick_size = 2
+        elif price >= 5000 and price < 9995:
+            tick_size = 5
+
+    elif market == 'US':
+        if price < 1:
+            tick_size = 0.0001
+        else:
+            tick_size = 0.001
+
+    return tick_size
\ No newline at end of file
